#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces

# import packages
from onf_website.reference.forest.schema import ForestsListField
from zope.interface import Interface, Attribute, invariant, Invalid
from zope.schema import Bool, Set, Choice

from onf_website import _


HEARING_INFO_KEY = 'onf_website.hearing'


class IHearingInfo(Interface):
    """Hearing information interface"""

    can_inherit = Attribute("Can inherit location?")

    inherit = Bool(title=_("Inherit location?"),
                   required=False,
                   default=True)

    no_inherit = Bool(title=_("Don't inherit location?"),
                      required=False,
                      default=False)

    targets = Set(title=_("Targets"),
                  description=_("Who is concerned by this content?"),
                  value_type=Choice(vocabulary='ONF targets'),
                  required=False)

    targets_index = Attribute("Targets list used for indexing")

    private = Bool(title=_("Restricted access?"),
                   description=_("If 'yes', access will be restricted to selected groups"),
                   required=False,
                   default=False)

    private_groups = Set(title=_("Authorized groups"),
                         description=_("List of groups having access to this content"),
                         value_type=Choice(vocabulary='PyAMS local groups'),
                         required=False)

    source = Choice(title=_("Original site"),
                    description=_("Main source site of current content"),
                    required=False,
                    vocabulary='PyAMS shared sites')

    source_index = Attribute("Source value used for indexing")

    diffusion_sites = Set(title=_("Diffusion sites"),
                          description=_("Other diffusion sites"),
                          value_type=Choice(vocabulary='PyAMS shared sites'),
                          required=False)

    diffusion_sites_index = Attribute("Diffusion sites list used for indexing")

    use_location = Bool(title=_("Use location settings"),
                        required=False,
                        default=True)

    ignore_location = Bool(title=_("Ignore location settings"),
                           required=False,
                           default=False)

    departments = Set(title=_("Other departments"),
                      value_type=Choice(vocabulary='ONF departments'),
                      required=False)

    departments_forests = Bool(title=_("Extend to all forests of selected departments"),
                               required=False,
                               default=False)

    domanials_only = Bool(title=_("Only extend to domanial forests"),
                          required=False,
                          default=False)

    communals_only = Bool(title=_("Only extend to communal forests"),
                          required=False,
                          default=False)

    @invariant
    def check_forests(self):
        if (self.domanials_only or self.communals_only) and not self.departments_forests:
            raise Invalid(_("You should check for domanials or communal forests only if you extend forests selection "
                            "to forests of selected departments!"))
        if self.domanials_only and self.communals_only:
            raise Invalid(_("You can extend forests selection to domanial OR communal forests, not both!"))

    forests = ForestsListField(title=_("Other forests"),
                               required=False)


class IHearingTarget(Interface):
    """Hearing target interface"""

    handle_hearing_targets = Bool(title="Handle hearing targets",
                                  required=True,
                                  default=True)

    handle_hearing_privacy = Bool(title="Handle hearing privacy?",
                                  required=True,
                                  default=True)

    handle_hearing_source = Bool(title="Handle hearing source?",
                                 required=True,
                                 default=True)

    handle_hearing_scope = Bool(title="Handle hearing scope?",
                                required=True,
                                default=True)


VIEW_HEARING_SETTINGS_KEY = 'onf_website.view.hearing'


class IViewHearingSettings(Interface):
    """View hearing settings"""

    select_context_targets = Bool(title=_("Select context targets?"),
                                  description=_("If 'yes', targets will be extracted from context hearing"),
                                  required=True,
                                  default=False)

    targets = Set(title=_("Other targets"),
                  value_type=Choice(vocabulary='ONF targets'),
                  required=False)

    def get_targets(self, context):
        """Get all targets from given context"""

    def get_targets_index(self, context):
        """Get all targets index values from given context"""

    select_context_source = Bool(title=_("Select context source site?"),
                                 description=_("If 'yes', source site will be extracted from context hearing"),
                                 required=True,
                                 default=False)

    source_sites = Set(title=_("Other source sites"),
                       value_type=Choice(vocabulary='PyAMS shared sites'),
                       required=False)

    def get_source_sites(self, context):
        """Get all source sites from given context"""

    def get_source_sites_index(self, context):
        """Get all source sites index values from given context"""

    select_context_sites = Bool(title=_("Select context diffusion sites?"),
                                description=_("If 'yes', diffusion sites will be extracted from context hearing"),
                                required=True,
                                default=False)

    diffusion_sites = Set(title=_("Other diffusion sites"),
                          value_type=Choice(vocabulary='PyAMS shared sites'),
                          required=False)

    def get_diffusion_sites(self, context):
        """Get all diffusion sites from given context"""

    def get_diffusion_sites_index(self, context):
        """Get all diffusion sites index values from given context"""
