#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from hypatia.interfaces import ICatalog
from onf_website.component.hearing.interfaces import IViewHearingSettings, IHearingInfo, VIEW_HEARING_SETTINGS_KEY
from pyams_content.shared.common.interfaces import ISharedSite
from pyams_content.shared.view.interfaces import IWfView, IViewQueryParamsExtension, IViewQueryEsParamsExtension
from zope.annotation.interfaces import IAnnotations
from zope.intid.interfaces import IIntIds

# import packages
from elasticsearch_dsl import Q
from hypatia.query import Any
from persistent import Persistent
from pyams_catalog.query import and_
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyams_utils.registry import get_utility, query_utility
from pyramid.threadlocal import get_current_registry
from zope.container.contained import Contained
from zope.interface import implementer
from zope.lifecycleevent import ObjectCreatedEvent
from zope.location import locate
from zope.schema.fieldproperty import FieldProperty


@implementer(IViewHearingSettings)
class ViewHearingSettings(Persistent, Contained):
    """View hearing settings"""

    select_context_targets = FieldProperty(IViewHearingSettings['select_context_targets'])
    targets = FieldProperty(IViewHearingSettings['targets'])
    select_context_source = FieldProperty(IViewHearingSettings['select_context_source'])
    source_sites = FieldProperty(IViewHearingSettings['source_sites'])
    select_context_sites = FieldProperty(IViewHearingSettings['select_context_sites'])
    diffusion_sites = FieldProperty(IViewHearingSettings['diffusion_sites'])

    def get_targets(self, context):
        targets = set()
        if self.select_context_targets:
            hearing = IHearingInfo(context, None)
            if hearing is not None:
                targets |= hearing.targets or set()
        if self.targets:
            targets |= self.targets
        return targets

    def get_targets_index(self, context):
        intids = get_utility(IIntIds)
        return [intids.register(target) for target in self.get_targets(context)]

    def get_source_sites(self, context):
        sites = set()
        if self.select_context_source:
            hearing = IHearingInfo(context, None)
            if hearing is not None:
                sites.add(hearing.source)
        if self.source_sites:
            sites |= self.source_sites
        return sites

    def get_source_sites_index(self, context):
        intids = get_utility(IIntIds)
        sites = [query_utility(ISharedSite, name=site) for site in self.get_source_sites(context)]
        return [intids.register(site) for site in sites if site is not None]

    def get_diffusion_sites(self, context):
        sites = set()
        if self.select_context_sites:
            hearing = IHearingInfo(context, None)
            if hearing is not None:
                sites |= hearing.diffusion_sites or set()
        if self.diffusion_sites:
            sites |= self.diffusion_sites
        return sites

    def get_diffusion_sites_index(self, context):
        intids = get_utility(IIntIds)
        sites = [query_utility(ISharedSite, name=site) for site in self.get_diffusion_sites(context)]
        return [intids.register(site) for site in sites if site is not None]


@adapter_config(context=IWfView, provides=IViewHearingSettings)
def ViewHearingSettingsFactory(view):
    """View hearing settings factory"""
    annotations = IAnnotations(view)
    settings = annotations.get(VIEW_HEARING_SETTINGS_KEY)
    if settings is None:
        settings = annotations[VIEW_HEARING_SETTINGS_KEY] = ViewHearingSettings()
        get_current_registry().notify(ObjectCreatedEvent(settings))
        locate(settings, view, '++view:hearing++')
    return settings


@adapter_config(name='hearing', context=IWfView, provides=IViewQueryParamsExtension)
class ViewHearingQueryParamsExtension(ContextAdapter):
    """View hearing query params extension"""

    weight = 50

    def get_params(self, context):
        catalog = get_utility(ICatalog)
        settings = IViewHearingSettings(self.context)
        params = None
        # check targets
        targets = settings.get_targets_index(context)
        if targets:
            params = and_(params, Any(catalog['targets'], targets))
        # check source sites
        sites = settings.get_source_sites_index(context)
        if sites:
            params = and_(params, Any(catalog['source_site'], sites))
        # check diffusion sites
        sites = settings.get_diffusion_sites_index(context)
        if sites:
            params = and_(params, Any(catalog['diffusion_sites'], sites))
        return params


@adapter_config(name='hearing', context=IWfView, provides=IViewQueryEsParamsExtension)
class ViewHearingQueryEsParamsExtension(ContextAdapter):
    """View hearing query params extension for Elasticsearch"""

    weight = 50

    def get_es_params(self, context):
        settings = IViewHearingSettings(self.context)
        params = []
        # check targets
        targets = settings.get_targets_index(context)
        if targets:
            params.append(Q('terms', **{'hearing.targets': targets}))
        # check sources
        sources = settings.get_source_sites_index(context)
        if sources:
            params.append(Q('terms', **{'hearing.source_site': sources}))
        # check diffusion sites
        sites = settings.get_diffusion_sites_index(context)
        if sites:
            params.append(Q('terms', **{'hearing.diffusion_sites': sites}))
        if params:
            return Q('bool', must=params)
